**----------------------------------------------------------------------------**
**  Program.....: STRU.PRG                                                    **
**  Function....: Display / Print / Code / Save  the Structure of a DBF       **
**  Client......: In-house                                                    **
**  Language....: Written in Clipper S'87                                     **
**  Written.....: February 1989                                               **
    Version     = "1.20"
    Copyright   = "(c) 1989 R. Stribling"
**  Programmer..: Robert S. Stribling                                         **
**----------------------------------------------------------------------------**

*------------------------------ VERIFY PARAMETERS -----------------------------*
PARAMETERS mDBFile

IF PCOUNT() = 0
  ? "*** Please Specify a DBF File ***"
  ?
  QUIT
ENDIF
IF .NOT. FILE(mDBFile)
  ? "*** DBF File Not Found ***"
  ?
  QUIT
ENDIF

*------------------------------ SET CLIPPER ENVIRONMENT -----------------------*
SAVE SCREEN
SET SCOREBOARD OFF
SET CURSOR OFF
SET COLOR TO W+/N,N/W, N
CLEAR

*------------------------------ Define Constants ------------------------------*
CR_LF    = CHR(13)+CHR(10)
mDBFile = UPPER(mDBFile)
StrLen   = LEN(mDBFile)-4
DBFName  = LEFT(mDBFile,StrLen)
StruName = DBFName + ".STR"
StruTxt  = DBFName + ".TXT"
GenFile  = DBFName + ".GEN"

*------------------------------ Define FUNCTION KEYS --------------------------*
SET KEY -1 TO List2Prn                  && F2
SET KEY -2 TO List2File                 && F3
SET KEY -3 TO RenStru                   && F4
SET KEY -4 TO GenCode                   && F5

*------------------------------ DATABASE SETUP --------------------------------*
IF FILE("DBF_STR1.DBF")
  ERASE DBF_STR1.DBF
ENDIF
IF FILE("DBF_STR2.DBF")
  ERASE DBF_STR2.DBF
ENDIF
IF FILE("DBF_STR3.DBF")
  ERASE DBF_STR3.DBF
ENDIF

SELECT 0
USE &mDBFile ALIAS DBFile
COPY STRUCTURE EXTENDED TO DBF_STR1

SELECT 0
CREATE DBF_STR2                         &&  Create temporary Structure DBF
USE DBF_STR2
COPY STRUCTURE EXTENDED TO DBF_STR3
USE
ERASE DBF_STR2.DBF

SELECT 0
USE DBF_STR3

APPEND BLANK
REPLACE FIELD_NAME WITH "FLD_NBR"
REPLACE FIELD_TYPE WITH "N"
REPLACE FIELD_LEN  WITH 3
REPLACE FIELD_DEC  WITH 0
APPEND BLANK
REPLACE FIELD_NAME WITH "TYPE_DESC"
REPLACE FIELD_TYPE WITH "C"
REPLACE FIELD_LEN  WITH 9
REPLACE FIELD_DEC  WITH 0
APPEND BLANK
REPLACE FIELD_NAME WITH "START"
REPLACE FIELD_TYPE WITH "N"
REPLACE FIELD_LEN  WITH 3
REPLACE FIELD_DEC  WITH 0
APPEND BLANK
REPLACE FIELD_NAME WITH "END"
REPLACE FIELD_TYPE WITH "N"
REPLACE FIELD_LEN  WITH 3
REPLACE FIELD_DEC  WITH 0

CREATE DBF_STRU FROM DBF_STR3
USE
ERASE DBF_STR3.DBF

SELECT 0
USE DBF_STRU
APPEND FROM DBF_STR1

SELECT DBFile
USE
ERASE DBF_STR1.DBF
*---------------------- End of: DATABASE SETUP --------------------------------*

*------------------------------ CALCULATE START & END POSITIONS ---------------*
SELECT DBF_STRU
GO TOP

m->FLD_NBR = 0
m->START   = 0
m->END     = 0

DO WHILE .NOT. EOF()

  m->FLD_NBR  = m->FLD_NBR + 1
  m->START    = m->END     + 1
  m->END      = m->END     + FIELD_LEN
  REPLACE FLD_NBR WITH m->FLD_NBR
  REPLACE START   WITH m->START
  REPLACE END     WITH m->END
  DO CASE
    CASE FIELD_TYPE = "C"
      REPLACE TYPE_DESC WITH "Character"
    CASE FIELD_TYPE = "N"
      REPLACE TYPE_DESC WITH "Numeric"
    CASE FIELD_TYPE = "D"
      REPLACE TYPE_DESC WITH "Date"
    CASE FIELD_TYPE = "L"
      REPLACE TYPE_DESC WITH "Logical"
    CASE FIELD_TYPE = "M"
      REPLACE TYPE_DESC WITH "Memo"
    OTHERWISE
      REPLACE TYPE_DESC WITH FIELD_TYPE
  ENDCASE
  SKIP 1

ENDDO

USE                             &&  CLOSE TO WRITE TO DISK
USE DBF_STRU                    &&  Reopen for use
*---------------------- End of: CALCULATE START & END POSITIONS ---------------*

*============================== MAIN LINE =====================================*
ScrnUDF()                               &&  Call Screen Function

SET COLOR TO W+/B
@ 00,00 SAY SPACE(80)
@ 00,00 SAY "DBF Structure Utility " + Version
@ 00,59 SAY Copyright
SET COLOR TO N/W
mText = "FILE: " + mDBFile
@ 00,CtrTxt(mText) SAY mText
SET COLOR TO W+/N,N/W

DECLARE aDBF_Flds[07],aDBF_Cols[07]
aDBF_Flds[01] = "FLD_NBR"
aDBF_Flds[02] = "FIELD_NAME"
aDBF_Flds[03] = "TYPE_DESC"
aDBF_Flds[04] = "FIELD_LEN"
aDBF_Flds[05] = "FIELD_DEC"
aDBF_Flds[06] = "START"
aDBF_Flds[07] = "END"

aDBF_Cols[01] = "Field"
aDBF_Cols[02] = "Field Name"
aDBF_Cols[03] = "Type"
aDBF_Cols[04] = "Width"
aDBF_Cols[05] = "Dec"
aDBF_Cols[06] = "Start"
aDBF_Cols[07] = "End"

GO TOP
DBEDIT(02,01,22,78,aDBF_Flds,"","",aDBF_Cols)

IF FILE("DBF_STRU.DBF")
  ERASE DBF_STRU.DBF
ENDIF

SET CURSOR ON
RESTORE SCREEN
QUIT
*====================== End of: MAIN LINE =====================================*

********************************************************************************
*  User Defined Functions  UDF()'s                    (Listed Alphabetically)  *
********************************************************************************

*------------------------------ FUNCTION CursorStat ---------------------------*
*  UDF to restore previous Cursor ON/OFF Stat
*  -  Use with:  CursorOn = ISCURSOR()
*------------------------------------------------------------------------------*
FUNCTION CursorStat

IF CursorOn
  SET CURSOR ON
ELSE
  SET CURSOR OFF
ENDIF
RETURN (.T.)
*---------------------- End of: FUNCTION CursorStat ---------------------------*

*------------------------------ FUNCTION CtrTxt -------------------------------*
*  Center Text on the Display Screen UDF
*  - Returns the starting column position
*------------------------------------------------------------------------------*
FUNCTION CtrTxt

PARAMETERS mString
RETURN INT((80 - LEN(mString)) /2)
*---------------------- End of: FUNCTION CtrTxt -------------------------------*

*------------------------------ FUNCTION GenCode ------------------------------*
*  F5  - GenCode        Generate Clipper Function to Create DBF
*------------------------------------------------------------------------------*
FUNCTION GenCode

PRIVATE mPCol

SET PRINTER TO &GenFile                 &&  Opens Output file
SET PRINT ON
SET DEVICE TO PRINT
SET CONSOLE OFF

@ PROW()+1, 0 SAY "*------------------------------ FUNCTION C_" +DBFName
mPCol = PCOL()
@ PROW()  ,mPCol+1 SAY REPLICATE("-",78-mPCol) + "*"
@ PROW()+1, 0 SAY "*  UDF to Create " +mDBFile
@ PROW()+1, 0 SAY "*  Returns with the New DBF created, written to disk, & closed"
@ PROW()+1, 0 SAY "*  Code Generated by: STRU - the DBF Structure Utility, version "+ Version
@ PROW()+1, 0 SAY "*" +REPLICATE("-",78) +"*"
@ PROW()+1, 0 SAY "FUNCTION C_" +DBFName

@ PROW()+2, 0 SAY "SELECT 0"
@ PROW()  ,40 SAY "&&  Open next work area"
@ PROW()+1, 0 SAY "CREATE " +StruName
@ PROW()  ,40 SAY "&&  CREATE (& USE) a temp Structure DBF"
@ PROW()+1, 0 SAY ""

GO TOP
DO WHILE .NOT. EOF()

  @ PROW()+1, 0 SAY 'APPEND BLANK'
  @ PROW()+1, 0 SAY 'REPLACE FIELD_NAME WITH "' +RTRIM(FIELD_NAME) +'"'
  @ PROW()+1, 0 SAY 'REPLACE FIELD_TYPE WITH "' +FIELD_TYPE +'"'
  @ PROW()+1, 0 SAY 'REPLACE FIELD_LEN  WITH ' +LTRIM(STR(FIELD_LEN))
  @ PROW()+1, 0 SAY 'REPLACE FIELD_DEC  WITH ' +LTRIM(STR(FIELD_DEC))
  SKIP 1

ENDDO

@ PROW()+2, 0 SAY "USE"
@ PROW()  ,40 SAY "&&  Close temporary DBF & write to disk"

@ PROW()+2, 0 SAY "SELECT 0"
@ PROW()  ,40 SAY "&&  Open next work area"
@ PROW()+1, 0 SAY "CREATE " +DBFName +" FROM " +StruName
@ PROW()  ,40 SAY "&&  CREATE (& USE) New DBF"
@ PROW()+1, 0 SAY "USE"
@ PROW()  ,40 SAY "&&  Close New DBF & write to disk"

@ PROW()+2, 0 SAY 'IF FILE("' +StruName +'")'
@ PROW()  ,40 SAY "&&  Erase Temporary Structure DBF"
@ PROW()+1, 0 SAY "  ERASE " +StruName
@ PROW()+1, 0 SAY "ENDIF"

@ PROW()+2, 0 SAY "RETURN(.T.)"
@ PROW()+1, 0 SAY "*---------------------- End of: FUNCTION C_" +DBFName
@ PROW()  ,mPCol+1 SAY REPLICATE("-",78-mPCol) + "*"
@ PROW()+2, 0 SAY ""

SET CONSOLE ON
SET DEVICE TO SCREEN
SET PRINT OFF
SET PRINTER TO                          &&  Closes file

COMMIT                                  &&  Write DBFs in all work areas to disk

RETURN(.T.)
*---------------------- End of: FUNCTION GenCode ------------------------------*

*------------------------------ FUNCTION HELP ---------------------------------*
FUNCTION HELP

PARAMETERS mProc, mLine, mVar

PRIVATE mRow, mCol, CursorOn

IF mProc = "HELP"
  RETURN(.T.)
ENDIF

CursorOn = ISCURSOR()                   &&  UDF to check CURSOR ON/OFF status 
mRow = ROW()                            &&  Save Current Cursor Position
mCol = COL()
LastCol_H = SETCOLOR()
SAVE SCREEN TO SCRN_H

SET COLOR TO W+/B                       &&  Brt White on Blue
CLEAR SCREEN
@ 00,00 TO 24,79                        &&  Single line box
SET COLOR TO W+/R                       &&  Brt White on Red
mText = " Help Text for STRU " +Version +" "
@ 00,CtrTxt(mText) SAY mText
SET COLOR TO W+/B                       &&  Brt White on Blue

TEXT



   Esc - EXIT           Exit to DOS

   F1  - HELP           This HELP Screen

   F2  - PRINT          Send DBF Structure to default Printer

   F3  - Prn-> .TXT     Write DBF Structure to "&StruTxt"

   F4  - Save as .STR   Save DBF Structure File as "&StruName"

   F5  - GenCode        Generate Clipper Function to Create "&mDBFile"








   Press any key to continue...
ENDTEXT
INKEY(0)

@ mRow,mCol SAY ""                      &&  Restore Cursor Position
RESTORE SCREEN FROM SCRN_H
SETCOLOR(LastCol_H)
CursorStat()                            &&  Restore previous Cursor ON/OFF Stat

RETURN(.T.)
*---------------------- End of: FUNCTION HELP ---------------------------------*

*------------------------------ FUNCTION List2File ----------------------------*
*  F3  - Prn-> .TXT     Write DBF Structure to "&StruTxt"
*------------------------------------------------------------------------------*
FUNCTION List2File

IF FILE(StruTxt)
  mXXX    = StruTxt
  IF .NOT. Over_Rite()
    RETURN(.T.)
  ENDIF
ENDIF
SET PRINTER TO &StruTxt                 &&  Opens file
SET PRINT ON
PrintStru()
SET PRINT OFF
SET PRINTER TO                          &&  Closes Output file

RETURN(.T.)
*---------------------- End of: FUNCTION List2File ----------------------------*

*------------------------------ FUNCTION List2Prn -----------------------------*
*  F2  - PRINT          Calls UDF that sends DBF Structure to default Printer
*------------------------------------------------------------------------------*
FUNCTION List2Prn

PrintStru()

RETURN(.T.)
*---------------------- End of: FUNCTION List2Prn -----------------------------*

*------------------------------ FUNCTION Over_Rite ----------------------------*
*  UDF to verify over write if file exists
*------------------------------------------------------------------------------*
FUNCTION Over_Rite

LastCol_0 = SETCOLOR()
WinBuff = SAVESCREEN(24,00,24,79)
SET COLOR TO W+/B,N/W                   &&  Bright White on Blue
Ans = "Y"
@ 24,00 SAY SPACE(80)
@ 24,00 SAY "File " +mXXX +" already exists...Overwrite? (Y/N) " GET Ans PICTURE "@!"
SET CURSOR ON
READ
SET CURSOR OFF

IF Ans <> UPPER("Y")
  Ret_Val = .F.
ELSE
  Ret_Val = .T.
ENDIF
RESTSCREEN(24,00,24,79,WinBuff)
SETCOLOR(LastCol_0)
RETURN(Ret_Val)
*---------------------- End of: FUNCTION Over_Rite ----------------------------*

*------------------------------ FUNCTION PrintStru ----------------------------*
*  UDF to send DBF Structure to default Printer
*------------------------------------------------------------------------------*
FUNCTION PrintStru

SET DEVICE TO PRINT
SET CONSOLE OFF

@ PROW(),PCOL() SAY CR_LF
@ PROW(),PCOL() SAY "DBF Structure Utility "+ Version + CR_LF
@ PROW(),PCOL() SAY " Structure for database: " + mDBFile + CR_LF
@ PROW(),PCOL() SAY "  Field  Field Name  Type       Width  Dec  Start  End"
LIST ALL FIELD_NAME,"", TYPE_DESC, "  ", FIELD_LEN,"", FIELD_DEC, "  ", START, "", END  TO PRINT
@ PROW(),PCOL() SAY CR_LF
SUM ALL FIELD_LEN TO mLen_Tot
@ PROW(),PCOL() SAY "** Total **                " +STR(mLen_Tot) +" (*" +LTRIM(STR(mLen_Tot+1)) +")"
@ PROW(),PCOL() SAY CR_LF + CR_LF

SET CONSOLE ON
SET DEVICE TO SCREEN
RETURN(.T.)
*---------------------- End of: FUNCTION PrintStru ----------------------------*

*------------------------------ FUNCTION RenStru ------------------------------*
*  F4  - Save as .STR   Save DBF Structure File as "&StruName"
*------------------------------------------------------------------------------*
FUNCTION RenStru

IF FILE("DBF_STRU.DBF")
  IF FILE(StruName)
    mXXX    = StruName
    IF Over_Rite()
      ERASE &StruName
    ELSE
      RETURN(.T.)
    ENDIF
  ENDIF
ENDIF
RENAME DBF_STRU.DBF &StruName
RETURN(.T.)
*---------------------- End of: FUNCTION RenStru ------------------------------*

*------------------------------ FUNCTION ScrnUDF ------------------------------*
*  UDF to paint main screen
*------------------------------------------------------------------------------*
FUNCTION ScrnUDF

SET COLOR TO W+/N,N/W
@ 01,00 TO 23,79                        &&  Single Line Box
@ 03,00 SAY ""
@ 03,79 SAY ""

SET COLOR TO +W/B                       &&  Bright White on Blue
mText = " Esc = Exit  F1 Help  F2 Print  F3 Prn-> .TXT  F4 Save as .STR  F5 GenCode "
@ 24,00 SAY mText

RETURN(.T.)
*---------------------- End of: FUNCTION ScrnUDF ------------------------------*
*** EOF ***
